/* 
  Copyright (C) 2000 Pawel A. Gajda (mis@k2.net.pl)
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License published by
  the Free Software Foundation (see file COPYING for details).
*/

/*
  $Id: test_match.c,v 1.19 2005/05/15 15:56:04 mis Exp $
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>          /* for PATH_MAX */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <trurl/nassert.h>
#include <trurl/nmalloc.h>

#include "i18n.h"
#include "log.h"
#include "pkg.h"
#include "misc.h"
#include "capreq.h"
#include "pm/pm.h"
#include "pm/rpm/pm_rpm.h"

int test_pmdb(int argc, char *argv[]) 
{
    
    char buf[PATH_MAX], tmp[PATH_MAX];
    struct pm_ctx *pmctx;
    struct pkgdb *db;
    struct capreq *req;

    
    pmmodule_init();
    
    pmctx = pm_new("rpm");

    db = pkgdb_open(pmctx, "/tmp/dupa/blada", NULL, O_RDONLY, NULL);
    n_assert(db == NULL);
}


int test_match(int argc, char *argv[])
{
    struct pkg *pkg;
    struct capreq *req;
    int epoch = 0;
    unsigned rels[] = { REL_EQ, REL_EQ | REL_GT, REL_EQ | REL_LT,
                           REL_GT, REL_LT, 0};

    
    char *versions[] = {"1.0", "1.1", NULL};
    char *relases[] = {"1", "2", NULL};
    int i, j, k;
        
    for (epoch=0; epoch<1; epoch++) {
        printf("\nEpoch: %d\n", epoch);
        i = 0;
        while (versions[i]) {
            j = 0;
            while (relases[j]) {
                struct capreq *cap;
                
                pkg = pkg_new("poldek", epoch, versions[i], relases[j], NULL, NULL);
                cap = capreq_new(NULL,
                                 "poldek", epoch, versions[i++], relases[j++],
                                 REL_EQ, 0);
                k = 0;
                while (rels[k] > 0) {
                    int rc1, rc2;
                    char evr[255];

                    snprintf(evr, sizeof(evr), "1:1.1");
                    req = capreq_new_evr("poldek", n_strdup(evr), rels[k++], 0);

                    rc1 = pkg_match_req(pkg, req, 1) ? 1:0;
                    rc2 = cap_match_req(cap, req, 1) ? 1:0;
                    printf("P %s[%d] match %s -> %s\n", pkg_snprintf_s(pkg),
                           pkg->epoch, capreq_snprintf_s(req),
                           rc1 ? "YES" : "NO");

                    printf("C %s match %s -> %s\n\n", capreq_snprintf_s(cap),
                           capreq_snprintf_s0(req),
                           rc2 ? "YES" : "NO");
                    n_assert(rc1 == rc2);
                }
            }
        }
    }
    return 0;
}

int test_match_(int epoch, char *ver, char *rel, char *evr, int relation) 
{
    struct pkg *pkg;
    struct capreq *req;
    int rc1;
    
    pkg = pkg_new("poldek", epoch, ver, rel,  NULL, NULL);

    req = capreq_new_evr("poldek", n_strdup(evr), relation, 0);

    rc1 = pkg_match_req(pkg, req, 1) ? 1:0;
    printf("P %s match %s ? %s\n", pkg_evr_snprintf_s(pkg),
           capreq_snprintf_s(req), rc1 ? "YES" : "NO");
    return 1;
}


int test_match2(void) 
{
    printf("\n");
    test_match_(0, "1.2", "1", ":1.1", REL_GT);
    test_match_(3, "1.2", "1", "0:1.1", REL_GT);
    test_match_(0, "1.2", "1", "3:1.1", REL_GT);
    printf("\n");
    test_match_(0, "1.2", "1", "1:1.1", REL_GT);
    test_match_(1, "1.2", "1", "0:1.1", REL_GT);

    printf("\n");
    test_match_(0, "1.2", "1", "0:1.2", REL_EQ);
    test_match_(3, "1.2", "1", "0:1.2", REL_EQ);
    test_match_(0, "1.2", "1", "3:1.2", REL_EQ);
    printf("\n");
    test_match_(0, "1.2", "1", "1:1.2", REL_EQ);
    test_match_(1, "1.2", "1", "0:1.2", REL_EQ);

    printf("\n");
    test_match_(0, "1.2", "1", "1:2.2", REL_EQ | REL_GT);
    test_match_(1, "1.2", "1", "1:1.2", REL_EQ | REL_GT);
    test_match_(1, "1.2", "1", "2:1.2", REL_EQ | REL_GT);
    test_match_(2, "1.2", "1", "2:1.2", REL_EQ | REL_GT);
    test_match_(3, "1.2", "1", "2:1.2", REL_EQ | REL_GT);
    return 0;
}

#if 0
int test_expand_env(int argc, char *argv[])
{
    int i;

    for (i=1; i<argc; i++) {
        char buf[PATH_MAX]; 
        printf("%s -> %s\n", argv[i], expand_env_vars(buf, sizeof(buf), argv[i]));
    }
        
    return 0;
}
#endif

int test_xmatch(int epoch, char *ver, char *rel, char *evr, int relation) 
{
    struct pkg *pkg;
    struct capreq *req;
    int rc1;
    
    pkg = pkg_new("poldek", epoch, ver, rel,  NULL, NULL);

    req = capreq_new_evr("poldek", n_strdup(evr), relation, 0);

    rc1 = pkg_xmatch_req(pkg, req, POLDEK_MA_PROMOTE_EPOCH) ? 1:0;
    printf("P %s match %s ? %s\n", pkg_evr_snprintf_s(pkg),
           capreq_snprintf_s(req), rc1 ? "YES" : "NO");
    return 1;
}

const char *maflags_snprintf_s(unsigned maflags) 
{
    static char buf[128];
    int n = 0, nsave;

    if (maflags == 0)
        return "";
    
    n += n_snprintf(&buf[n], sizeof(buf) - n, "[maf=");
    nsave = n;
    
    if (maflags & POLDEK_MA_PROMOTE_VERSION)
        n += n_snprintf(&buf[n], sizeof(buf) - n, "v,");
    
    if (maflags & POLDEK_MA_PROMOTE_REQEPOCH)
        n += n_snprintf(&buf[n], sizeof(buf) - n, "re,");

    if (maflags & POLDEK_MA_PROMOTE_CAPEPOCH)
        n += n_snprintf(&buf[n], sizeof(buf) - n, "ce,");

    if (n == nsave)
        return "";
    
    buf[n - 1] = ']';
    return buf;
}


int do_test_pkgmatch(char *pkgevr, char *evr, int relation, unsigned maflags) 
{
    struct pkg *pkg;
    struct capreq *req;
    const char *ver, *rel;
    int rc;
    int32_t epoch;

    if (!poldek_util_parse_evr(n_strdup(pkgevr), &epoch, &ver, &rel))
        return -1;
    
    pkg = pkg_new("poo", epoch, ver, rel,  NULL, NULL);
    req = capreq_new_evr("poo", n_strdup(evr), relation, 0);

    rc = pkg_xmatch_req(pkg, req, maflags) ? 1 : 0;
    printf("  %s match%s %s => %s\n", pkg_evr_snprintf_s(pkg),
           maflags_snprintf_s(maflags), capreq_snprintf_s(req),
           rc ? "YES" : "NO");
    return rc;
}

int do_test_capmatch(char *capevr, char *evr, int relation, unsigned maflags, 
                     int expected) 
{
    struct capreq *cap, *req;
    int rc;

    if (capevr)
        cap = capreq_new_evr("coo", n_strdup(capevr), REL_EQ, 0);
    else
        cap = capreq_new(NULL, "coo", 0, NULL, NULL, 0, 0);
    printf("evr = %s\n", evr);
    req = capreq_new_evr("coo", n_strdup(evr), relation, 0);
    capreq_snprintf_s(req);
    
    rc = cap_xmatch_req(cap, req, maflags) ? 1 : 0;
    capreq_snprintf_s(cap);
    capreq_snprintf_s(req);
    printf("dupa\n");
    printf("  %s match%s %s => %s\n", capreq_snprintf_s(cap),
           maflags_snprintf_s(maflags), capreq_snprintf_s0(req),
           rc ? "YES" : "NO");

    capreq_free(cap);
    capreq_free(req);
    return rc;
}


int main(int argc, char *argv[]) 
{
    rpmdb db;
    struct capreq *req;

    test_pmdb(argc, argv);
    
    req = capreq_new_evr("aa", n_strdup("1.2"), REL_EQ, 0);
    capreq_snprintf_s(req);
    
    //test_expand_env(argc, argv);
    //{"0:1.2-1", "1:1.2",  REL_EQ | REL_GT, POLDEK_MA_PROMOTE_EPOCH, 0 },
    do_test_capmatch(NULL, "1.2", 0, POLDEK_MA_PROMOTE_VERSION, 0);
    exit(0);
    poldek_log_init(NULL, stdout, "aa");
    pm_rpm_init();
    db = pm_rpm_opendb(NULL, NULL, "/var/lib/rpm", "/", O_RDONLY, NULL);
    sleep(20);
    pm_rpm_closedb(db);
    sleep(30);
    return 0;
}
